/**
 * @file      sh30f9xx_sb0_interrupt.c
 * @brief     Main Interrupt Service Routines.
 *            This file provides template for all exceptions handler and 
 *            peripherals interrupt service routine.
 * @author    sinowealth software group ()
 * @date      2025-04-02
 * @version   V1.0.1
 * @copyright Copyright (c) 1994 - 2025 SINOWEALTH Electronic Ltd.
 * 
 * @attention
 * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY ON SINOWEALTH'S
 * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
 * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
 * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
 * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
 * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
 * CONNECTION WITH THEIR PRODUCTS.
 * @par Change log:
 * <table>
 * <tr><th>Date        <th>Version  <th>Author    <th>Description
 * <tr><td>2025-03-10  <td>1.0      <td>bruce     <td>Create initial version
 * </table>
 */

/* Includes ------------------------------------------------------------------*/
#include "sh30f9xx_sb0_libcfg.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
#if defined(__clang__) 
    void NMI_Handler (void);
    void HardFault_Handler (void) __attribute__((noreturn));
    void SVC_Handler (void);
    void PendSV_Handler (void);
    void SysTick_Handler (void);
    void WWDT_Handler (void);
    void BOD_Handler (void);
    void RCC_Handler (void);
    void EXTI0_Handler (void);
    void EXTI1_Handler (void);
    void EXTI2_Handler (void);
    void EXTI3_Handler (void);
    void ADC_Handler (void);
    void TWI0_Handler (void);
    void TWI1_Handler (void);
    void PCA0_Handler (void);
    void PCA1_Handler (void);
    void PCA2_Handler (void);
    void PCA3_Handler (void);
    void EXTI9_4_Handler (void);
    void UART0_Handler (void);
    void SPI0_Handler (void);
    void SPI1_Handler (void);
    void UART1_Handler (void);
    void UART2_Handler (void);
    void UART3_Handler (void);
    void EXTI15_10_Handler (void);
    void UART4_Handler (void);
    void UART5_Handler (void);
    void PWM0_Handler (void);
    void PWM1_Handler (void);
    void PWM2_Handler (void);
    void PWM3_Handler (void);
    void TIM0_Handler (void);
    void TIM1_FIFO0_Handler (void);
    void TIM2_FIFO1_Handler (void);
    void TIM3_FIFO2_Handler (void);
#endif
/******************************************************************************/
/*            Cortex-M0+ Processor Exceptions Handlers                        */
/******************************************************************************/

/**
 * @brief  This function handles NMI interrupt request.
 * @param   None
 * @retval  None
 * @note    After the occurrence of CSM, although the system clock has 
 *          automatically switched to HSI/8, in order to maintain system stability, 
 *          the AHB bus division coefficient must be set to 8. After the failed clock 
 *          is restored (HSERDY flag is set), clear the CSM flag, and the system clock
 *          will switch back to the previous abnormal clock. Then, change the AHB bus 
 *          division coefficient back to the original set value.
 */
void NMI_Handler (void)
{   
    static uint8_t clockDivision = 0;
    if (RCC_GetFlagStatus(RCC_CSM_HSE_IF) == SH_SET)
    {
        /* The system clock has automatically switched to HSI/8 */
       
        /*Backup AHB bus division coefficient*/
        clockDivision = RCC_GetAHBClockDivision();
        /*The AHB bus division coefficient must be set to 8 divisions*/
        RCC_SetAHBClockDivision(RCC_AHB_DIV8);
        
        /* This is to prevent the occurrence of the HSI clock being manually turned off when HSE is used as a system */
        RCC_HSIOnOff(SH_ON);
        while(RCC_GetHSIReadyFlag() == SH_RESET);
        /* Switch the clock source to HSE */
        RCC_SetSystemClockSource(RCC_SYS_SRC_HSI);

        /* HSE must be closed first and then opened again */
        RCC_HSEOnOff(SH_OFF);
        __NOP();
        RCC_HSEOnOff(SH_ON);

        /* Next, we are preparing to restore the clock by querying it */
        while (RCC_GetHSEReadyFlag() == SH_RESET)
        {
            HWDT_ReloadCounter();
        }
        /* Clear the CSM interrupt hold bit */
        RCC_ClearITPendingBit(RCC_CICCLR_CSMC);
        /* Switch the clock source to HSE */
        RCC_SetSystemClockSource(RCC_SYS_SRC_HSE);
        /* Restore the AHB bus to its pre abnormal value */
        RCC_SetAHBClockDivision((RCC_AHB_DIVISION_TypeDef)clockDivision);

    }
}

/**
 * @brief  This function handles HardFault interrupt request.
 * @param  None
 * @retval None
 */
void HardFault_Handler (void)
{
    /* Go to infinite loop when HardFault exception occurs */
    while (1)
    {
        HWDT->CLR = 0xAAAA;
    }
}

/**
 * @brief  This function handles SVC interrupt request.
 * @param  None
 * @retval None
 */
void SVC_Handler (void)
{

}

/**
 * @brief  This function handles PendSV interrupt request.
 * @param  None
 * @retval None
 */
void PendSV_Handler (void)
{

}

/**
 * @brief  This function handles SysTick interrupt request.
 * @param  None
 * @retval None
 */
void SysTick_Handler (void)
{

}

/**
 * @brief  This function handles WWDT interrupt request.
 * @param  None
 * @retval None
 */
void WWDT_Handler (void)
{

}

/**
 * @brief  This function handles BOD interrupt request.
 * @param  None
 * @retval None
 */
void BOD_Handler (void)
{

}

/**
 * @brief  This function handles RCC interrupt request.
 * @param  None
 * @retval None
 */
void RCC_Handler (void)
{

}

/**
 * @brief  This function handles EXTI0 interrupt request.
 * @param  None
 * @retval None
 */
void EXTI0_Handler (void)
{

}

/**
 * @brief  This function handles EXTI1 interrupt request.
 * @param  None
 * @retval None
 */
void EXTI1_Handler (void)
{

}

/**
 * @brief  This function handles EXTI2 interrupt request.
 * @param  None
 * @retval None
 */
void EXTI2_Handler (void)
{

}

/**
 * @brief  This function handles EXTI3 interrupt request.
 * @param  None
 * @retval None
 */
void EXTI3_Handler (void)
{

}

/**
 * @brief  This function handles ADC interrupt request.
 * @param  None
 * @retval None
 */
void ADC_Handler (void)
{

}

/**
 * @brief  This function handles TWI0 interrupt request.
 * @param  None
 * @retval None
 */
void TWI0_Handler (void)
{

}

/**
 * @brief  This function handles TWI1 interrupt request.
 * @param  None
 * @retval None
 */
void TWI1_Handler (void)
{

}

/**
 * @brief  This function handles PCA0 interrupt request.
 * @param  None
 * @retval None
 */
void PCA0_Handler (void)
{

}

/**
 * @brief  This function handles PCA1 interrupt request.
 * @param  None
 * @retval None
 */
void PCA1_Handler (void)
{

}

/**
 * @brief  This function handles PCA2 interrupt request.
 * @param  None
 * @retval None
 */
void PCA2_Handler (void)
{

}

/**
 * @brief  This function handles PCA3 interrupt request.
 * @param  None
 * @retval None
 */
void PCA3_Handler (void)
{

}

/**
 * @brief  This function handles EXTI9_4 interrupt request.
 * @param  None
 * @retval None
 */
void EXTI9_4_Handler (void)
{

}

/**
 * @brief  This function handles UART0 interrupt request.
 * @param  None
 * @retval None
 */
void UART0_Handler (void)
{

}

/**
 * @brief  This function handles SPI0 interrupt request.
 * @param  None
 * @retval None
 */
void SPI0_Handler (void)
{

}

/**
 * @brief  This function handles SPI1 interrupt request.
 * @param  None
 * @retval None
 */
void SPI1_Handler (void)
{

}

/**
 * @brief  This function handles UART1 interrupt request.
 * @param  None
 * @retval None
 */
void UART1_Handler (void)
{

}

/**
 * @brief  This function handles UART2 interrupt request.
 * @param  None
 * @retval None
 */
void UART2_Handler (void)
{

}

/**
 * @brief  This function handles UART3 interrupt request.
 * @param  None
 * @retval None
 */
void UART3_Handler (void)
{

}

/**
 * @brief  This function handles EXTI15_10 interrupt request.
 * @param  None
 * @retval None
 */
void EXTI15_10_Handler (void)
{

}

/**
 * @brief  This function handles UART4 interrupt request.
 * @param  None
 * @retval None
 */
void UART4_Handler (void)
{

}

/**
 * @brief  This function handles UART5 interrupt request.
 * @param  None
 * @retval None
 */
void UART5_Handler (void)
{

}

/**
 * @brief  This function handles PWM0 interrupt request.
 * @param  None
 * @retval None
 */
void PWM0_Handler (void)
{

}

/**
 * @brief  This function handles PWM1 interrupt request.
 * @param  None
 * @retval None
 */
void PWM1_Handler (void)
{

}

/**
 * @brief  This function handles PWM2 interrupt request.
 * @param  None
 * @retval None
 */
void PWM2_Handler (void)
{

}

/**
 * @brief  This function handles PWM3 interrupt request.
 * @param  None
 * @retval None
 */
void PWM3_Handler (void)
{

}

/**
 * @brief  This function handles TIM0 interrupt request.
 * @param  None
 * @retval None
 */
void TIM0_Handler (void)
{

}

/**
 * @brief  This function handles TIM1_FIFO0 interrupt request.
 * @param  None
 * @retval None
 */
void TIM1_FIFO0_Handler (void)
{

}

/**
 * @brief  This function handles TIM2_FIFO1 interrupt request.
 * @param  None
 * @retval None
 */
void TIM2_FIFO1_Handler (void)
{

}

/**
 * @brief  This function handles TIM3_FIFO2 interrupt request.
 * @param  None
 * @retval None
 */
void TIM3_FIFO2_Handler (void)
{

}

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/
